@@ -99,7 +99,6 @@ gwr.mixed <- function(formula, data, regression.points, fixed.vars,intercept.fix
   colnames(x1) <- colnames(x)[-idx.fixed]
   colnames(x2) <- colnames(x)[idx.fixed]
   #y <- as.matrix(y, nrow = dp.n)
-  
   model <- gwr.mixed.2(x1, x2, y, dp.locat, out.loc=rp.locat, adaptive=adaptive, bw=bw,
                         kernel=kernel, p=p, theta=theta, longlat=longlat,dMat)                     
   res <- list()
@@ -141,7 +140,7 @@ gwr.mixed <- function(formula, data, regression.points, fixed.vars,intercept.fix
 #       SDF <- SpatialPointsDataFrame(coords=rp.locat, data=mgwr.df, proj4string=CRS(p4s), match.ID=F)
    
    res$SDF <- SDF
-   if (diagnostic)
+   if (hatmatrix)
    {
       gwr.fitted <- function(x,b) apply(x*b,1,sum)
       edf <- gwr.mixed.trace(x1, x2, y, dp.locat, adaptive=adaptive, bw=bw,
@@ -168,7 +167,7 @@ gwr.mixed <- function(formula, data, regression.points, fixed.vars,intercept.fix
    res}
 
 ##Mixed GWR
-gwr.mixed.2 <- function(x1, x2, y, loc, out.loc=loc, adaptive=F, bw=sqrt(var(loc[,1])+var(loc[,2])),
+gwr.mixed.2 <- function(x1, x2, y, loc, out.loc, adaptive=F, bw=sqrt(var(loc[,1])+var(loc[,2])),
                kernel="bisquare", p=2, theta=0, longlat=F,dMat)
 {
   #gwr.fitted <- function(x,b) apply(x*b,1,sum)
@@ -176,24 +175,27 @@ gwr.mixed.2 <- function(x1, x2, y, loc, out.loc=loc, adaptive=F, bw=sqrt(var(loc
    
    ncols.2 <- dim(x2)[2]
    x3 <- NULL
-
+   if(missing(out.loc))
+     dMat1 <- dMat
+   else
+     dMat1 <- gw.dist(loc, p=p, theta = theta, longlat = longlat)
    for (i in 1:ncols.2)
    {
       m.temp <-gwr.q(x1, x2[,i], loc, adaptive=adaptive, bw=bw,
-                  kernel=kernel, p=p, theta=theta, longlat=longlat,dMat=dMat)
+                  kernel=kernel, p=p, theta=theta, longlat=longlat, dMat = dMat1)
       x3 <- cbind(x3,x2[,i]-gw.fitted(x1,m.temp))
    }
    colnames(x3) <- colnames(x2)
    m.temp <-gwr.q(x1, y, loc, adaptive=adaptive, bw=bw,
-                  kernel=kernel, p=p, theta=theta, longlat=longlat,dMat=dMat)
+                  kernel=kernel, p=p, theta=theta, longlat=longlat, dMat = dMat1)
    y2 <- y - gw.fitted(x1,m.temp)
    
-
    model2 <-gwr.q(x3, y2, loc, adaptive=TRUE, bw=1.0e6, kernel="boxcar",
-                  p=p, theta=theta, longlat=longlat,dMat=dMat)
+                  p=p, theta=theta, longlat=longlat, dMat = dMat1)
    fit2 <- gw.fitted(x2,model2)
    model1 <-gwr.q(x1, y-fit2, loc, out.loc=out.loc,adaptive=adaptive, bw=bw,
                   kernel=kernel, p=p, theta=theta, longlat=longlat,dMat=dMat)
+   
    model2 <-gwr.q(x3, y2, loc,out.loc=out.loc, adaptive=TRUE, bw=1.0e6, kernel="boxcar",
                   p=p, theta=theta, longlat=longlat,dMat=dMat)
    list(local=model1,global=model2)
@@ -203,7 +205,7 @@ gwr.mixed.2 <- function(x1, x2, y, loc, out.loc=loc, adaptive=F, bw=sqrt(var(loc
 # Fix the column names for x3
 #
 
-gwr.mixed.trace <- function(x1, x2, y, loc, out.loc=loc, adaptive=F, bw=sqrt(var(loc[,1])+var(loc[,2])),
+gwr.mixed.trace <- function(x1, x2, y, loc, out.loc, adaptive=F, bw=sqrt(var(loc[,1])+var(loc[,2])),
                kernel="bisquare", p=2, theta=0, longlat=F,dMat)
   {gwr.fitted <- function(x,b) apply(x*b,1,sum)
    e.vec <- function(m,n) as.numeric(m == 1:n)
@@ -215,11 +217,14 @@ gwr.mixed.trace <- function(x1, x2, y, loc, out.loc=loc, adaptive=F, bw=sqrt(var
 
    ncols.2 <- dim(x2)[2]
    #n.items <- length(y)
-
+   if(missing(out.loc))
+     dMat1 <- dMat
+   else
+     dMat1 <- gw.dist(loc, p=p, theta = theta, longlat = longlat)
    x3 <- NULL
    for (i in 1:ncols.2)
      {m.temp <-gwr.q(x1, x2[,i], loc, adaptive=adaptive, bw=bw,
-                  kernel=kernel, p=p, theta=theta, longlat=longlat,dMat=dMat) 
+                  kernel=kernel, p=p, theta=theta, longlat=longlat,dMat=dMat1) 
       x3 <- cbind(x3,x2[,i]-gw.fitted(x1,m.temp))}
     
    colnames(x3) <- colnames(x2)
@@ -228,11 +233,11 @@ gwr.mixed.trace <- function(x1, x2, y, loc, out.loc=loc, adaptive=F, bw=sqrt(var
    for (i in 1:dp.n)
      {
        m.temp <-gwr.q(x1, e.vec(i,dp.n), loc, adaptive=adaptive, bw=bw,
-                  kernel=kernel, p=p, theta=theta, longlat=longlat,dMat=dMat)  
+                  kernel=kernel, p=p, theta=theta, longlat=longlat,dMat=dMat1)  
        y2 <- e.vec(i,dp.n) - gwr.fitted(x1,m.temp)
 
        model2 <-gwr.q(x3,y2, loc,  adaptive=TRUE, bw=1.0e6, kernel="boxcar",
-                p=p, theta=theta, longlat=longlat,dMat=dMat)
+                p=p, theta=theta, longlat=longlat,dMat=dMat1)
        fit2 <- gwr.fitted(x2,model2)
        if(DM.given)
        {
@@ -335,7 +340,10 @@ gwr.q <- function(x, y, loc, out.loc=loc, adaptive=F, bw=sqrt(var(loc[,1])+var(l
      DM.given <- F
   else
      DM.given <- T
-  rp.n <- nrow(out.loc)
+  if(missing(out.loc))
+    rp.n <- nrow(loc)
+  else
+    rp.n <- nrow(out.loc)
   var.n <- ncol(x)
   betas <- matrix(nrow=rp.n, ncol=var.n)
   for (i in 1:rp.n)
